@@ -30,7 +30,9 @@
 
 #define UPDATE_QUALITY_DELAY 500
 #define UPDATE_VISIBILITY_DELAY 100
-#define N_HEADER_BAR_BUTTONS 2
+#define N_HEADER_BAR_BUTTONS 3
+#define HIDE_OVERVIEW_TIMEOUT 2 /* in seconds */
+#define OVERLAY_MARGIN 10
 
 
 static void gth_viewer_page_interface_init (GthViewerPageInterface *iface);
@@ -92,6 +94,7 @@ struct _GthImageViewerPagePrivate {
 	GtkWidget         *buttons[N_HEADER_BAR_BUTTONS];
 	gboolean           pointer_on_viewer;
 	gboolean           pointer_on_overview;
+	guint              hide_overview_id;
 };
 
 
@@ -342,11 +345,31 @@ update_image_quality_if_required (GthImageViewerPage *self)
 
 
 static gboolean
-update_visibility_cb (gpointer user_data)
+hide_overview_after_timeout (gpointer data)
 {
-	GthImageViewerPage *self = user_data;
-	gboolean            visible;
+	GthImageViewerPage *self = data;
 
+	if (self->priv->hide_overview_id != 0)
+		g_source_remove (self->priv->hide_overview_id);
+	self->priv->hide_overview_id = 0;
+
+	if (! self->priv->pointer_on_overview) {
+		if (gtk_revealer_get_child_revealed (GTK_REVEALER (self->priv->overview_revealer)))
+			gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->overview_revealer), FALSE);
+	}
+
+	return FALSE;
+}
+
+
+static gboolean
+update_overview_visibility_now (gpointer user_data)
+{
+	GthImageViewerPage *self;
+	gboolean            overview_visible;
+	gboolean            revealed;
+
+	self = GTH_IMAGE_VIEWER_PAGE (user_data);
 	if (! self->priv->active)
 		return FALSE;
 
@@ -355,10 +378,13 @@ update_visibility_cb (gpointer user_data)
 		self->priv->update_visibility_id = 0;
 	}
 
-	visible = self->priv->pointer_on_overview || (self->priv->pointer_on_viewer && gth_image_viewer_has_scrollbars (GTH_IMAGE_VIEWER (self->priv->viewer)));
-	if (visible != gtk_revealer_get_child_revealed (GTK_REVEALER (self->priv->overview_revealer))) {
-		gtk_widget_show (self->priv->overview_revealer);
-		gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->overview_revealer), visible);
+	overview_visible = self->priv->pointer_on_overview || (self->priv->pointer_on_viewer && gth_image_viewer_has_scrollbars (GTH_IMAGE_VIEWER (self->priv->viewer)));
+	gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->overview_revealer), overview_visible);
+
+	if (overview_visible) {
+		if (self->priv->hide_overview_id != 0)
+			g_source_remove (self->priv->hide_overview_id);
+		self->priv->hide_overview_id = g_timeout_add_seconds (HIDE_OVERVIEW_TIMEOUT, hide_overview_after_timeout, self);
 	}
 
 	return FALSE;
@@ -368,24 +394,26 @@ update_visibility_cb (gpointer user_data)
 static void
 update_overview_visibility (GthImageViewerPage *self)
 {
-	if (self->priv->update_visibility_id != 0)
-		g_source_remove (self->priv->update_visibility_id);
-	self->priv->update_visibility_id = g_timeout_add (UPDATE_VISIBILITY_DELAY, update_visibility_cb, self);
+	if (gtk_revealer_get_child_revealed (GTK_REVEALER (self->priv->overview_revealer))) {
+		if (self->priv->update_visibility_id != 0) {
+			g_source_remove (self->priv->update_visibility_id);
+			self->priv->update_visibility_id = 0;
+		}
+	}
+
+	if (self->priv->update_visibility_id == 0)
+		self->priv->update_visibility_id = g_timeout_add (UPDATE_VISIBILITY_DELAY,
+								  update_overview_visibility_now,
+								  self);
 }
 
 
 static void
-viewer_zoom_changed_cb (GtkWidget          *widget,
-			GthImageViewerPage *self)
+update_zoom_info (GthImageViewerPage *self)
 {
 	double  zoom;
 	char   *text;
 
-	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (self));
-	update_image_quality_if_required (self);
-	self->priv->pointer_on_viewer = TRUE;
-	update_overview_visibility (self);
-
 	zoom = gth_image_viewer_get_zoom (GTH_IMAGE_VIEWER (self->priv->viewer));
 	text = g_strdup_printf ("  %d%%  ", (int) (zoom * 100));
 	gth_statusbar_set_secondary_text (GTH_STATUSBAR (gth_browser_get_statusbar (self->priv->browser)), text);
@@ -394,11 +422,25 @@ viewer_zoom_changed_cb (GtkWidget          *widget,
 }
 
 
+static void
+viewer_zoom_changed_cb (GtkWidget          *widget,
+			GthImageViewerPage *self)
+{
+	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (self));
+	update_image_quality_if_required (self);
+	self->priv->pointer_on_viewer = TRUE;
+	update_overview_visibility (self);
+	update_zoom_info (self);
+}
+
+
 static void
 viewer_image_changed_cb (GtkWidget          *widget,
 			 GthImageViewerPage *self)
 {
+	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (self));
 	update_overview_visibility (self);
+	update_zoom_info (self);
 }
 
 
@@ -411,40 +453,6 @@ viewer_button_press_event_cb (GtkWidget          *widget,
 }
 
 
-static gboolean
-viewer_button_enter_notify_event_cb (GtkWidget *widget,
-				     GdkEvent  *event,
-				     gpointer   user_data)
-{
-	GthImageViewerPage *self = user_data;
-
-	if (widget == self->priv->overview)
-		self->priv->pointer_on_overview = TRUE;
-	else if (widget == self->priv->viewer)
-		self->priv->pointer_on_viewer = TRUE;
-	update_overview_visibility (self);
-
-	return FALSE;
-}
-
-
-static gboolean
-viewer_button_leave_notify_event_cb (GtkWidget *widget,
-				     GdkEvent  *event,
-				     gpointer   user_data)
-{
-	GthImageViewerPage *self = user_data;
-
-	if (widget == self->priv->overview)
-		self->priv->pointer_on_overview = gth_image_overview_get_scrolling_is_active (GTH_IMAGE_OVERVIEW (self->priv->overview));
-	else if (widget == self->priv->viewer)
-		self->priv->pointer_on_viewer = FALSE;
-	update_overview_visibility (self);
-
-	return FALSE;
-}
-
-
 static gboolean
 viewer_popup_menu_cb (GtkWidget          *widget,
 		      GthImageViewerPage *self)
@@ -555,6 +563,65 @@ viewer_unrealize_cb (GtkWidget *widget,
 }
 
 
+static gboolean
+image_navigator_get_child_position_cb	(GtkOverlay   *overlay,
+					 GtkWidget    *widget,
+					 GdkRectangle *allocation,
+					 gpointer      user_data)
+{
+	GthImageViewerPage *self = GTH_IMAGE_VIEWER_PAGE (user_data);
+	GtkAllocation       main_alloc;
+	gboolean            allocation_filled = FALSE;
+
+	gtk_widget_get_allocation (gtk_bin_get_child (GTK_BIN (overlay)), &main_alloc);
+	gtk_widget_get_preferred_width (widget, NULL, &allocation->width);
+	gtk_widget_get_preferred_height (widget, NULL, &allocation->height);
+
+	if (widget == self->priv->overview_revealer) {
+		allocation->x = main_alloc.width - allocation->width - OVERLAY_MARGIN;
+		allocation->y = OVERLAY_MARGIN;
+		allocation_filled = TRUE;
+	}
+
+	return allocation_filled;
+}
+
+
+static gboolean
+overview_motion_notify_event_cb (GtkWidget      *widget,
+				 GdkEventMotion *event,
+				 gpointer        data)
+{
+	GthImageViewerPage *self = data;
+
+	if (self->priv->hide_overview_id != 0) {
+		g_source_remove (self->priv->hide_overview_id);
+		self->priv->hide_overview_id = 0;
+	}
+
+	self->priv->pointer_on_viewer = TRUE;
+	if (widget == self->priv->overview)
+		self->priv->pointer_on_overview = TRUE;
+	update_overview_visibility (data);
+
+	return FALSE;
+}
+
+
+static gboolean
+overview_leave_notify_event_cb (GtkWidget *widget,
+				GdkEvent  *event,
+				gpointer   data)
+{
+	GthImageViewerPage *self = data;
+
+	if (widget == self->priv->overview)
+		self->priv->pointer_on_overview = FALSE;
+
+	return FALSE;
+}
+
+
 static void
 pref_zoom_quality_changed (GSettings *settings,
 			   char      *key,
@@ -739,44 +806,6 @@ paint_comment_over_image_func (GthImageViewer *image_viewer,
 }
 
 
-static gboolean
-image_navigator_get_child_position_cb	(GtkOverlay   *overlay,
-					 GtkWidget    *widget,
-					 GdkRectangle *allocation,
-					 gpointer      user_data)
-{
-	GthImageViewerPage *self = GTH_IMAGE_VIEWER_PAGE (user_data);
-
-	if (widget == self->priv->overview_revealer) {
-		GtkAllocation main_alloc;
-
-		gtk_widget_get_allocation (gtk_bin_get_child (GTK_BIN (overlay)), &main_alloc);
-		gtk_widget_get_preferred_width (widget, NULL, &allocation->width);
-		gtk_widget_get_preferred_height (widget, NULL, &allocation->height);
-		allocation->x = main_alloc.width - allocation->width - 10;
-		allocation->y = 10;
-
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-
-static void
-overview_revealer_notify_child_revealed_cb (GObject    *gobject,
-					    GParamSpec *pspec,
-					    gpointer    user_data)
-{
-	GthImageViewerPage *self = user_data;
-	gboolean            visible;
-
-	visible = gtk_revealer_get_child_revealed (GTK_REVEALER (self->priv->overview_revealer));
-	if (! visible)
-		gtk_widget_hide (self->priv->overview_revealer);
-}
-
-
 static void
 gth_image_viewer_page_real_activate (GthViewerPage *base,
 				     GthBrowser    *browser)
@@ -796,16 +825,23 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 			gth_browser_add_header_bar_button (browser,
 							   GTH_BROWSER_HEADER_SECTION_VIEWER_VIEW,
 							   "view-zoom-original-symbolic",
-							   NULL,
+							   _("Set to actual size"),
 							   "win.image-zoom-100",
 							   NULL);
 	self->priv->buttons[1] =
 			gth_browser_add_header_bar_button (browser,
 							   GTH_BROWSER_HEADER_SECTION_VIEWER_VIEW,
 							   "view-zoom-fit-symbolic",
-							   NULL,
+							   _("Fit to window"),
 							   "win.image-zoom-fit",
 							   NULL);
+	self->priv->buttons[2] =
+			gth_browser_add_header_bar_button (browser,
+							   GTH_BROWSER_HEADER_SECTION_VIEWER_VIEW,
+							   "view-zoom-fit-width-symbolic",
+							   _("Fit to width"),
+							   "win.image-zoom-fit-width",
+							   NULL);
 	gth_window_add_accelerators (GTH_WINDOW (browser), accelerators, G_N_ELEMENTS (accelerators));
 
 	self->priv->preloader = gth_browser_get_image_preloader (browser);
@@ -831,14 +867,6 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 				"button_press_event",
 				G_CALLBACK (viewer_button_press_event_cb),
 				self);
-	g_signal_connect_after (G_OBJECT (self->priv->viewer),
-				"enter-notify-event",
-				G_CALLBACK (viewer_button_enter_notify_event_cb),
-				self);
-	g_signal_connect_after (G_OBJECT (self->priv->viewer),
-				"leave-notify-event",
-				G_CALLBACK (viewer_button_leave_notify_event_cb),
-				self);
 	g_signal_connect_after (G_OBJECT (self->priv->viewer),
 				"scroll_event",
 				G_CALLBACK (viewer_scroll_event_cb),
@@ -869,27 +897,23 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 	gtk_widget_show (self->priv->image_navigator);
 
 	self->priv->overview_revealer = gtk_revealer_new ();
-	gtk_revealer_set_transition_duration (GTK_REVEALER (self->priv->overview_revealer), 500);
-	gtk_revealer_set_transition_type (GTK_REVEALER (self->priv->overview_revealer), GTK_REVEALER_TRANSITION_TYPE_CROSSFADE);
+	gtk_revealer_set_transition_duration (GTK_REVEALER (self->priv->overview_revealer), 200);
+	gtk_revealer_set_transition_type (GTK_REVEALER (self->priv->overview_revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN);
+	gtk_widget_show (self->priv->overview_revealer);
 	gtk_overlay_add_overlay (GTK_OVERLAY (self->priv->image_navigator), self->priv->overview_revealer);
 
-	g_signal_connect (G_OBJECT (self->priv->overview_revealer),
-			  "notify::child-revealed",
-			  G_CALLBACK (overview_revealer_notify_child_revealed_cb),
-			  self);
-
 	self->priv->overview = gth_image_overview_new (GTH_IMAGE_VIEWER (self->priv->viewer));
-	gtk_widget_add_events (self->priv->overview, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+	gtk_widget_add_events (self->priv->overview, GDK_POINTER_MOTION_HINT_MASK | GDK_LEAVE_NOTIFY_MASK);
 	gtk_widget_show (self->priv->overview);
 	gtk_container_add (GTK_CONTAINER (self->priv->overview_revealer), self->priv->overview);
 
 	g_signal_connect_after (G_OBJECT (self->priv->overview),
-				"enter-notify-event",
-				G_CALLBACK (viewer_button_enter_notify_event_cb),
+				"motion-notify-event",
+				G_CALLBACK (overview_motion_notify_event_cb),
 				self);
 	g_signal_connect_after (G_OBJECT (self->priv->overview),
 				"leave-notify-event",
-				G_CALLBACK (viewer_button_leave_notify_event_cb),
+				G_CALLBACK (overview_leave_notify_event_cb),
 				self);
 
 	gth_browser_set_viewer_widget (browser, self->priv->image_navigator);
@@ -925,7 +949,7 @@ gth_image_viewer_page_real_deactivate (GthViewerPage *base)
 		self->priv->buttons[i] = NULL;
 	}
 
-	g_object_unref (self->priv->preloader);
+	_g_object_unref (self->priv->preloader);
 	self->priv->preloader = NULL;
 	self->priv->active = FALSE;
 
@@ -1185,6 +1209,14 @@ gth_image_viewer_page_real_show_pointer (GthViewerPage *base,
 		gth_image_viewer_show_cursor (GTH_IMAGE_VIEWER (self->priv->viewer));
 	else if (gth_browser_get_is_fullscreen (self->priv->browser))
 		gth_image_viewer_hide_cursor (GTH_IMAGE_VIEWER (self->priv->viewer));
+
+	if (self->priv->hide_overview_id != 0) {
+		g_source_remove (self->priv->hide_overview_id);
+		self->priv->hide_overview_id = 0;
+	}
+
+	self->priv->pointer_on_viewer = show;
+	update_overview_visibility (self);
 }
 
 
@@ -1209,8 +1241,8 @@ gth_image_viewer_page_real_update_sensitivity (GthViewerPage *base)
 	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-in", zoom_enabled && (zoom < 100.0));
 
 	fit_mode = gth_image_viewer_get_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer));
-	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit", zoom_enabled && (fit_mode != GTH_FIT_SIZE_IF_LARGER));
-	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit-width", zoom_enabled && (fit_mode != GTH_FIT_WIDTH_IF_LARGER));
+	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit", zoom_enabled && (fit_mode != GTH_FIT_SIZE));
+	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit-width", zoom_enabled && (fit_mode != GTH_FIT_WIDTH));
 
 	_gth_image_viewer_page_update_paste_command_sensitivity (self, NULL);
 }
@@ -1561,6 +1593,10 @@ gth_image_viewer_page_finalize (GObject *obj)
 		g_source_remove (self->priv->update_visibility_id);
 		self->priv->update_visibility_id = 0;
 	}
+	if (self->priv->hide_overview_id != 0) {
+		g_source_remove (self->priv->hide_overview_id);
+		self->priv->hide_overview_id = 0;
+	}
 
 	g_object_unref (self->priv->settings);
 	g_object_unref (self->priv->history);
@@ -1605,16 +1641,26 @@ gth_viewer_page_interface_init (GthViewerPageInterface *iface)
 static void
 gth_image_viewer_page_init (GthImageViewerPage *self)
 {
+	int i;
+
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_IMAGE_VIEWER_PAGE, GthImageViewerPagePrivate);
 	self->priv->settings = g_settings_new (GTHUMB_IMAGE_VIEWER_SCHEMA);
+	self->priv->preloader = NULL;
+	self->priv->file_popup_merge_id = 0;
 	self->priv->history = gth_image_history_new ();
-	self->priv->last_loaded = NULL;
+	self->priv->file_data = NULL;
+	self->priv->active = FALSE;
 	self->priv->image_changed = FALSE;
 	self->priv->loading_image = FALSE;
+	self->priv->last_loaded = NULL;
 	self->priv->can_paste = FALSE;
 	self->priv->update_quality_id = 0;
 	self->priv->update_visibility_id = 0;
-	self->priv->active = FALSE;
+	for (i = 0; i < N_HEADER_BAR_BUTTONS; i++)
+		self->priv->buttons[i] = NULL;
+	self->priv->pointer_on_overview = FALSE;
+	self->priv->pointer_on_viewer = FALSE;
+	self->priv->hide_overview_id = 0;
 }
 
 
