@@ -24,6 +24,7 @@
 #include <glib/gi18n.h>
 #include <gthumb.h>
 #include <extensions/catalogs/gth-catalog.h>
+#include "actions.h"
 #include "gth-search-editor-dialog.h"
 #include "gth-search-task.h"
 
@@ -63,19 +64,21 @@ search_editor_dialog__response_cb (GtkDialog *dialog,
 
 
 void
-gth_browser_activate_action_edit_find (GtkAction  *action,
-				       GthBrowser *browser)
+gth_browser_activate_find (GSimpleAction *action,
+			   GVariant      *parameter,
+			   gpointer       user_data)
 {
-	GthSearch *search;
-	GtkWidget *dialog;
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	GthSearch  *search;
+	GtkWidget  *dialog;
 
 	search = gth_search_new ();
 	gth_search_set_folder (search, gth_browser_get_location (browser));
 	gth_search_set_recursive (search, TRUE);
 
 	dialog = gth_search_editor_dialog_new (_("Find"), search, GTK_WINDOW (browser));
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_FIND, GTK_RESPONSE_OK);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Find"), GTK_RESPONSE_OK);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (search_editor_dialog__response_cb),
@@ -135,8 +138,7 @@ search_update_buffer_ready_cb (void     **buffer,
 
 
 void
-gth_browser_activate_action_edit_search_update (GtkAction  *action,
-						GthBrowser *browser)
+gth_browser_update_search (GthBrowser *browser)
 {
 	GFile      *location;
 	SearchData *search_data;
