@@ -65,14 +65,6 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
-static void
-help_clicked_cb (GtkWidget  *widget,
-		 DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (data->dialog), "webalbums");
-}
-
-
 static void
 ok_clicked_cb (GtkWidget  *widget,
 	       DialogData *data)
@@ -209,6 +201,7 @@ ok_clicked_cb (GtkWidget  *widget,
 	gth_browser_exec_task (data->browser, task, FALSE);
 	gtk_widget_destroy (data->dialog);
 
+	g_object_unref (task);
 	g_free (image_attributes);
 	g_free (thumbnail_caption);
 	g_free (theme_name);
@@ -473,7 +466,7 @@ dlg_web_exporter (GthBrowser *browser,
 		destination = _g_settings_get_uri (data->settings, PREF_WEBALBUMS_DESTINATION);
 		if (destination == NULL)
 			destination = g_strdup (get_home_uri ());
-		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), destination);
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), destination);
 
 		g_free (destination);
 	}
@@ -488,10 +481,6 @@ dlg_web_exporter (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-        g_signal_connect (GET_WIDGET ("help_button"),
-                          "clicked",
-                          G_CALLBACK (help_clicked_cb),
-                          data);
 	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
