@@ -1096,7 +1096,7 @@ typedef struct {
 	GthFileData          *source;
 	GFile                *destination;
 	gboolean              move;
-	GFileCopyFlags        flags;
+	GthFileCopyFlags      flags;
 	int                   io_priority;
 	goffset               tot_size;
 	goffset               copied_size;
@@ -1112,6 +1112,7 @@ typedef struct {
 	GFile                *current_destination;
 	char                 *message;
 	GthOverwriteResponse  default_response;
+	gboolean              duplicating_file;
 
 	GList                *source_sidecars;  /* GFile list */
 	GList                *destination_sidecars;  /* GFile list */
@@ -1262,7 +1263,7 @@ copy_file__overwrite_dialog_response_cb (GtkDialog *dialog,
 
 			parent = g_file_get_parent (copy_file_data->current_destination);
 			new_destination = g_file_get_child_for_display_name (parent, gth_overwrite_dialog_get_filename (GTH_OVERWRITE_DIALOG (dialog)), NULL);
-			_g_copy_file_to_destination (copy_file_data, new_destination, 0);
+			_g_copy_file_to_destination (copy_file_data, new_destination, G_FILE_COPY_NONE);
 
 			g_object_unref (new_destination);
 			g_object_unref (parent);
@@ -1284,7 +1285,16 @@ copy_file_ready_cb (GObject      *source_object,
 
 	if (! g_file_copy_finish ((GFile *) source_object, result, &error)) {
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
-			if (copy_file_data->default_response != GTH_OVERWRITE_RESPONSE_ALWAYS_NO) {
+			if (copy_file_data->duplicating_file) {
+
+				/* the duplicated file already exists, try another one */
+
+				GFile *new_destination = _g_file_get_duplicated (copy_file_data->current_destination);
+				_g_copy_file_to_destination (copy_file_data, new_destination, G_FILE_COPY_NONE);
+
+				g_object_unref (new_destination);
+			}
+			else if (copy_file_data->default_response != GTH_OVERWRITE_RESPONSE_ALWAYS_NO) {
 				GtkWidget *dialog;
 
 				dialog = gth_overwrite_dialog_new (copy_file_data->source->file,
@@ -1319,7 +1329,7 @@ copy_file_ready_cb (GObject      *source_object,
 	_g_object_list_unref (copy_file_data->destination_sidecars);
 	copy_file_data->source_sidecars = NULL;
 	copy_file_data->destination_sidecars = NULL;
-	if (copy_file_data->flags && G_FILE_COPY_ALL_METADATA) {
+	if (copy_file_data->flags & GTH_FILE_COPY_ALL_METADATA) {
 		gth_hook_invoke ("add-sidecars", copy_file_data->source->file, &copy_file_data->source_sidecars);
 		gth_hook_invoke ("add-sidecars", copy_file_data->destination, &copy_file_data->destination_sidecars);
 		copy_file_data->source_sidecars = g_list_reverse (copy_file_data->source_sidecars);
@@ -1363,16 +1373,52 @@ copy_file_progress_cb (goffset  current_num_bytes,
 }
 
 
+static void
+_g_copy_file_to_destination_call_ready_callback (gpointer user_data)
+{
+	CopyFileData *copy_file_data = user_data;
+
+	copy_file_data->ready_callback (copy_file_data->default_response, NULL, copy_file_data->user_data);
+	copy_file_data_free (copy_file_data);
+}
+
+
 static void
 _g_copy_file_to_destination (CopyFileData   *copy_file_data,
 			     GFile          *destination,
 			     GFileCopyFlags  flags)
 {
+	if (copy_file_data->default_response == GTH_OVERWRITE_RESPONSE_ALWAYS_YES)
+		flags |= G_FILE_COPY_OVERWRITE;
+	if (copy_file_data->flags & GTH_FILE_COPY_ALL_METADATA)
+		flags |= G_FILE_COPY_ALL_METADATA;
+
 	_g_object_unref (copy_file_data->current_destination);
 	copy_file_data->current_destination = g_file_dup (destination);
 
-	if (copy_file_data->default_response == GTH_OVERWRITE_RESPONSE_ALWAYS_YES)
-		flags = G_FILE_COPY_OVERWRITE;
+	if (g_file_equal (copy_file_data->source->file, copy_file_data->current_destination)) {
+		GFile *old_destination;
+
+		if ((copy_file_data->move
+		    || (copy_file_data->default_response == GTH_OVERWRITE_RESPONSE_ALWAYS_YES)
+		    || ! (copy_file_data->flags & GTH_FILE_COPY_RENAME_SAME_FILE)))
+		{
+			call_when_idle (_g_copy_file_to_destination_call_ready_callback, copy_file_data);
+			return;
+		}
+
+		copy_file_data->duplicating_file = TRUE;
+
+		old_destination = copy_file_data->current_destination;
+		copy_file_data->current_destination = _g_file_get_duplicated (old_destination);
+		g_object_unref (old_destination);
+
+		/* duplicated files shouldn't be overwritten, if a duplicated
+		 * file already exists get another duplicated file and try
+		 * again. */
+		if (flags & G_FILE_COPY_OVERWRITE)
+			flags ^= G_FILE_COPY_OVERWRITE;
+	}
 
 	if (copy_file_data->progress_callback != NULL) {
 		GFile *destination_parent;
@@ -1396,6 +1442,9 @@ _g_copy_file_to_destination (CopyFileData   *copy_file_data,
 	if (g_file_info_get_file_type (copy_file_data->source->info) == G_FILE_TYPE_DIRECTORY) {
 		GError *error = NULL;
 
+		/* FIXME: use the async version, available since glib 2.38 */
+		/* FIXME: handle the GTH_FILE_COPY_RENAME_SAME_FILE flag for directories */
+
 		if (! g_file_make_directory (copy_file_data->current_destination, copy_file_data->cancellable, &error)) {
 			if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
 				g_clear_error (&error);
@@ -1407,7 +1456,7 @@ _g_copy_file_to_destination (CopyFileData   *copy_file_data,
 	else
 		g_file_copy_async (copy_file_data->source->file,
 				   copy_file_data->current_destination,
-				   copy_file_data->flags | flags,
+				   flags,
 				   copy_file_data->io_priority,
 				   copy_file_data->cancellable,
 				   copy_file_progress_cb,
@@ -1421,7 +1470,7 @@ static void
 _g_copy_file_async_private (GthFileData           *source,
 			    GFile                 *destination,
 			    gboolean               move,
-			    GFileCopyFlags         flags,
+			    GthFileCopyFlags       flags,
 			    GthOverwriteResponse   default_response,
 			    int                    io_priority,
 			    goffset                tot_size,
@@ -1455,7 +1504,7 @@ _g_copy_file_async_private (GthFileData           *source,
 	copy_file_data->user_data = user_data;
 	copy_file_data->default_response = default_response;
 
-	_g_copy_file_to_destination (copy_file_data, copy_file_data->destination, 0);
+	_g_copy_file_to_destination (copy_file_data, copy_file_data->destination, G_FILE_COPY_NONE);
 }
 
 
@@ -1463,7 +1512,7 @@ void
 _g_copy_file_async (GthFileData           *source,
 		    GFile                 *destination,
 		    gboolean               move,
-		    GFileCopyFlags         flags,
+		    GthFileCopyFlags       flags,
 		    GthOverwriteResponse   default_response,
 		    int                    io_priority,
 		    GCancellable          *cancellable,
@@ -1519,7 +1568,7 @@ typedef struct {
 	GthOverwriteResponse  default_response;
 
 	gboolean              move;
-	GFileCopyFlags        flags;
+	GthFileCopyFlags      flags;
 	int                   io_priority;
 	GCancellable         *cancellable;
 	ProgressCallback      progress_callback;
@@ -1652,12 +1701,6 @@ copy_data__copy_current_file (CopyData *copy_data)
 	}
 	destination = _g_file_get_destination (source->file, copy_data->source_base, copy_data->destination);
 
-	if (g_file_equal (source->file, destination)) {
-		g_object_unref (destination);
-		call_when_idle ((DataFunc) copy_data__copy_next_file, copy_data);
-		return;
-	}
-
 	_g_copy_file_async_private (source,
 				    destination,
 				    copy_data->move,
@@ -1712,7 +1755,7 @@ void
 _g_copy_files_async (GList                *sources, /* GFile list */
 		     GFile                *destination,
 		     gboolean              move,
-		     GFileCopyFlags        flags,
+		     GthFileCopyFlags      flags,
 		     GthOverwriteResponse  default_response,
 		     int                   io_priority,
 		     GCancellable         *cancellable,
@@ -1789,7 +1832,7 @@ _g_move_file (GFile                 *source,
 		return FALSE;
 	}
 
-	if (flags && G_FILE_COPY_ALL_METADATA == 0)
+	if ((flags & G_FILE_COPY_ALL_METADATA) == 0)
 		return TRUE;
 
 	/* move the metadata sidecars if requested */
