@@ -222,6 +222,7 @@ _gth_file_list_clear_queue (GthFileList *file_list)
 
 	if (file_list->priv->update_event != 0) {
 		g_source_remove (file_list->priv->update_event);
+		file_list->priv->update_event = 0;
 		file_list->priv->dirty = FALSE;
 	}
 
@@ -437,6 +438,8 @@ gth_file_list_init (GthFileList *file_list)
 	file_list->priv->visibles = NULL;
 	file_list->priv->visibility_changed = FALSE;
 	file_list->priv->thumbnailer_state.phase = THUMBNAILER_PHASE_INITIALIZE;
+	file_list->priv->dirty_event = 0;
+	file_list->priv->restart_thumb_update = 0;
 }
 
 
@@ -448,6 +451,11 @@ restart_thumb_update_cb (gpointer data)
 {
 	GthFileList *file_list = data;
 
+	if (file_list->priv->update_event != 0) {
+		g_source_remove (file_list->priv->update_event);
+		file_list->priv->update_event = 0;
+	}
+
 	if (file_list->priv->restart_thumb_update != 0) {
 		g_source_remove (file_list->priv->restart_thumb_update);
 		file_list->priv->restart_thumb_update = 0;
@@ -1608,6 +1616,12 @@ static gboolean
 start_thumbnail_job (gpointer user_data)
 {
 	ThumbnailJob *job = user_data;
+	GthFileList  *file_list = job->file_list;
+
+	if (file_list->priv->update_event != 0) {
+		g_source_remove (file_list->priv->update_event);
+		file_list->priv->update_event = 0;
+	}
 
 	job->started = TRUE;
 	gth_thumb_loader_load (job->loader,
