@@ -120,8 +120,7 @@ gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 	list = gth_file_source_vfs_add_uri (list, file_source, get_home_uri ());
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_PICTURES);
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_VIDEOS);
-	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DESKTOP);
-	/*list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DOCUMENTS);*/
+	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DOWNLOAD);
 	list = gth_file_source_vfs_add_uri (list, file_source, "file:///");
 
 	mounts = g_volume_monitor_get_mounts (g_volume_monitor_get ());
@@ -380,7 +379,7 @@ gth_file_source_vfs_copy (GthFileSource    *file_source,
 	_g_copy_files_async (file_list,
 			     destination->file,
 			     move,
-			     G_FILE_COPY_ALL_METADATA,
+			     GTH_FILE_COPY_ALL_METADATA | GTH_FILE_COPY_RENAME_SAME_FILE,
 			     GTH_OVERWRITE_RESPONSE_UNSPECIFIED,
 			     G_PRIORITY_DEFAULT,
 			     gth_file_source_get_cancellable (file_source),
