@@ -25,9 +25,11 @@
 #include <time.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include "dom.h"
 #include "glib-utils.h"
 #include "gth-filterbar.h"
 #include "gth-main.h"
+#include "gth-user-dir.h"
 
 enum {
 	ITEM_TYPE_NONE,
@@ -58,8 +60,10 @@ struct _GthFilterbarPrivate
 	GthTest      *test;
 	GtkWidget    *control_box;
 	GtkWidget    *control;
+	GtkWidget    *extra_area;
 	GtkTreeIter   current_iter;
 	gulong        filters_changed_id;
+	gulong        test_changed_id;
 };
 
 
@@ -78,8 +82,10 @@ gth_filterbar_finalize (GObject *object)
 
 	if (filterbar->priv != NULL) {
 		g_signal_handler_disconnect (gth_main_get_default_monitor (), filterbar->priv->filters_changed_id);
-		if (filterbar->priv->test != NULL)
+		if (filterbar->priv->test != NULL) {
+			g_signal_handler_disconnect (filterbar->priv->test, filterbar->priv->test_changed_id);
 			g_object_unref (filterbar->priv->test);
+		}
 		g_free (filterbar->priv);
 		filterbar->priv = NULL;
 	}
@@ -141,14 +147,6 @@ gth_filterbar_changed (GthFilterbar *filterbar)
 }
 
 
-static void
-close_button_clicked_cb (GtkWidget    *button,
-			 GthFilterbar *filterbar)
-{
-	g_signal_emit (filterbar, gth_filterbar_signals[CLOSE_BUTTON_CLICKED], 0);
-}
-
-
 static void
 _gth_filterbar_set_test_control (GthFilterbar *filterbar,
 				  GtkWidget    *control)
@@ -171,28 +169,36 @@ _gth_filterbar_set_test_control (GthFilterbar *filterbar,
 }
 
 
+static void
+test_changed_cb (GthTest      *test,
+		 GthFilterbar *filterbar)
+{
+	gth_filterbar_changed (filterbar);
+}
+
+
 static void
 _gth_filterbar_set_test (GthFilterbar *filterbar,
 			  GthTest      *test)
 {
 	if (filterbar->priv->test != NULL) {
+		if (filterbar->priv->test_changed_id != 0)
+			g_signal_handler_disconnect (filterbar->priv->test, filterbar->priv->test_changed_id);
 		g_object_unref (filterbar->priv->test);
 		filterbar->priv->test = NULL;
 	}
 
 	if (test != NULL) {
 		filterbar->priv->test = g_object_ref (test);
+		filterbar->priv->test_changed_id = g_signal_connect (test,
+								     "changed",
+								     G_CALLBACK (test_changed_cb),
+								     filterbar);
 		_gth_filterbar_set_test_control (filterbar, gth_test_create_control (filterbar->priv->test));
 	}
 	else
 		_gth_filterbar_set_test_control (filterbar, NULL);
-}
 
-
-static void
-test_changed_cb (GthTest      *test,
-		 GthFilterbar *filterbar)
-{
 	gth_filterbar_changed (filterbar);
 }
 
@@ -214,12 +220,6 @@ test_combo_box_changed_cb (GtkComboBox  *scope_combo_box,
 			    FILTER_COLUMN, &test,
 			    -1);
 
-	if (test != NULL)
-		g_signal_connect (test,
-				  "changed",
-				  G_CALLBACK (test_changed_cb),
-				  filterbar);
-
 	switch (item_type) {
 	case ITEM_TYPE_FILTER:
 		_gth_filterbar_set_test (filterbar, test);
@@ -281,7 +281,7 @@ update_filter_list (GthFilterbar *filterbar,
 		gtk_list_store_set (filterbar->priv->model, &iter,
 				    TYPE_COLUMN, ITEM_TYPE_FILTER,
 				    FILTER_COLUMN, test,
-				    NAME_COLUMN, gth_test_get_display_name(test),
+				    NAME_COLUMN, gth_test_get_display_name (test),
 				    -1);
 
 		if (g_strcmp0 (current_filter, gth_test_get_id (test)) == 0) {
@@ -301,7 +301,7 @@ update_filter_list (GthFilterbar *filterbar,
 	gtk_list_store_append (filterbar->priv->model, &iter);
 	gtk_list_store_set (filterbar->priv->model, &iter,
 			    TYPE_COLUMN, ITEM_TYPE_PERSONALIZE,
-			    NAME_COLUMN, _("Personalize..."),
+			    NAME_COLUMN, _("Personalize…"),
 			    -1);
 
 	if (no_filter_selected) {
@@ -338,8 +338,6 @@ gth_filterbar_construct (GthFilterbar *filterbar,
 {
 	GtkCellRenderer *renderer;
 	GtkWidget       *label;
-	GtkWidget       *button;
-	GtkWidget       *image;
 
 	gtk_box_set_spacing (GTK_BOX (filterbar), 6);
 	gtk_container_set_border_width (GTK_CONTAINER (filterbar), 2);
@@ -396,18 +394,10 @@ gth_filterbar_construct (GthFilterbar *filterbar,
 	filterbar->priv->control_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_widget_show (filterbar->priv->control_box);
 
-	/* close button */
+	/* extra widgets container */
 
-	button = gtk_button_new ();
-	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-	gtk_container_add (GTK_CONTAINER (button), image);
-	gtk_widget_hide (button);
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_widget_set_tooltip_text (button, _("Hide the filterbar"));
-	g_signal_connect (G_OBJECT (button),
-			  "clicked",
-			  G_CALLBACK (close_button_clicked_cb),
-			  filterbar);
+	filterbar->priv->extra_area = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_widget_show (filterbar->priv->extra_area);
 
 	/* view label */
 
@@ -426,7 +416,7 @@ gth_filterbar_construct (GthFilterbar *filterbar,
 	gtk_box_pack_start (GTK_BOX (filterbar), label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (filterbar), filterbar->priv->test_combo_box, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (filterbar), filterbar->priv->control_box, FALSE, FALSE, 0);
-	gtk_box_pack_end (GTK_BOX (filterbar), button, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (filterbar), filterbar->priv->extra_area, FALSE, FALSE, 0);
 }
 
 
@@ -450,3 +440,120 @@ gth_filterbar_get_test (GthFilterbar *filterbar)
 	else
 		return NULL;
 }
+
+
+void
+gth_filterbar_save_filter (GthFilterbar *filterbar,
+			   const char   *filename)
+{
+	char  *filter_description;
+	gsize  len;
+	GFile *filter_file;
+
+	if (filterbar->priv->test != NULL) {
+		DomDocument *doc;
+
+		doc = dom_document_new ();
+		dom_element_append_child (DOM_ELEMENT (doc), dom_domizable_create_element (DOM_DOMIZABLE (filterbar->priv->test), doc));
+		filter_description = dom_document_dump (doc, &len);
+
+		g_object_unref (doc);
+	}
+	else {
+		filter_description = g_strdup ("");
+		len = 0;
+	}
+	filter_file = gth_user_dir_get_file_for_write (GTH_DIR_CONFIG, GTHUMB_DIR, filename, NULL);
+	_g_file_write (filter_file, FALSE, 0, filter_description, len, NULL, NULL);
+
+	g_object_unref (filter_file);
+	g_free (filter_description);
+}
+
+
+static gboolean
+find_test_by_id (GthFilterbar  *filterbar,
+		 const char    *id,
+		 GthTest      **test,
+		 GtkTreeIter   *iter)
+{
+	g_return_val_if_fail (test != NULL, FALSE);
+	g_return_val_if_fail (iter != NULL, FALSE);
+
+	if (! gtk_tree_model_get_iter_first(GTK_TREE_MODEL (filterbar->priv->model), iter))
+		return FALSE;
+
+	do {
+		int item_type = ITEM_TYPE_NONE;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (filterbar->priv->model),
+				    iter,
+				    TYPE_COLUMN, &item_type,
+				    FILTER_COLUMN, test,
+				    -1);
+
+		if ((item_type == ITEM_TYPE_FILTER) && (*test != NULL) && (g_strcmp0 (gth_test_get_id (*test), id) == 0))
+			return TRUE;
+	}
+	while (gtk_tree_model_iter_next (GTK_TREE_MODEL (filterbar->priv->model), iter));
+
+	return FALSE;
+}
+
+
+void
+gth_filterbar_load_filter (GthFilterbar *filterbar,
+			   const char   *filename)
+{
+	GFile       *filter_file;
+	char        *buffer;
+	gsize        len;
+	DomDocument *doc;
+
+	filter_file = gth_user_dir_get_file_for_write (GTH_DIR_CONFIG, GTHUMB_DIR, filename, NULL);
+	if (! _g_file_load_in_buffer (filter_file, (void **) &buffer, &len, NULL, NULL)) {
+		g_object_unref (filter_file);
+		return;
+	}
+
+	if (buffer == NULL) {
+		g_object_unref (filter_file);
+		return;
+	}
+
+	doc = dom_document_new ();
+	if (dom_document_load (doc, buffer, len, NULL)) {
+		DomElement *node = DOM_ELEMENT (doc)->first_child;
+
+		if (node != NULL) {
+			GthTest     *test;
+			GtkTreeIter  iter;
+
+			if (find_test_by_id (filterbar,
+					     dom_element_get_attribute (node, "id"),
+					     &test,
+					     &iter))
+			{
+				dom_domizable_load_from_element (DOM_DOMIZABLE (test), node);
+
+				g_signal_handlers_block_by_func (filterbar->priv->test_combo_box, test_combo_box_changed_cb, filterbar);
+				gtk_combo_box_set_active_iter (GTK_COMBO_BOX (filterbar->priv->test_combo_box), &iter);
+				g_signal_handlers_unblock_by_func (filterbar->priv->test_combo_box, test_combo_box_changed_cb, filterbar);
+
+				filterbar->priv->current_iter = iter;
+				_gth_filterbar_set_test (GTH_FILTERBAR (filterbar), test);
+			}
+		}
+	}
+
+	g_object_unref (doc);
+	g_free (buffer);
+	g_object_unref (filter_file);
+}
+
+
+GtkWidget *
+gth_filterbar_get_extra_area (GthFilterbar *filterbar)
+{
+	return filterbar->priv->extra_area;
+}
