@@ -369,8 +369,10 @@ set_zoom (GthImageViewer *self,
 
 	_gth_image_viewer_update_image_area (self);
 	if (self->priv->update_image_after_zoom) {
+		g_signal_emit (G_OBJECT (self), gth_image_viewer_signals[IMAGE_CHANGED], 0);
 		gth_image_viewer_tool_image_changed (self->priv->tool);
 		self->priv->update_image_after_zoom = FALSE;
+		self->priv->skip_zoom_change = TRUE;
 	}
 	else
 		gth_image_viewer_tool_zoom_changed (self->priv->tool);
@@ -760,8 +762,7 @@ change_animation_frame (gpointer data)
 	gdk_pixbuf_animation_iter_advance (self->priv->iter, &self->priv->time);
 
 	_cairo_clear_surface (&self->priv->iter_surface);
-	self->priv->skip_zoom_change = TRUE;
-	gtk_widget_queue_resize (GTK_WIDGET (self));
+	gtk_widget_queue_draw (GTK_WIDGET (self));
 
 	return FALSE;
 }
@@ -1605,7 +1606,7 @@ _gth_image_viewer_content_changed (GthImageViewer *self,
 
 	if (better_quality)
 		g_signal_emit (G_OBJECT (self), gth_image_viewer_signals[BETTER_QUALITY], 0);
-	else
+	else if (! self->priv->zoom_enabled || (self->priv->zoom_change == GTH_ZOOM_CHANGE_KEEP_PREV))
 		g_signal_emit (G_OBJECT (self), gth_image_viewer_signals[IMAGE_CHANGED], 0);
 
 	if (! better_quality && self->priv->reset_scrollbars) {
