@@ -41,7 +41,7 @@
 #endif
 
 
-static GStaticMutex register_mutex = G_STATIC_MUTEX_INIT;
+static GMutex register_mutex;
 
 
 typedef struct {
@@ -206,12 +206,12 @@ gth_main_register_file_source (GType file_source_type)
 {
 	GObject *file_source;
 
-	g_static_mutex_lock (&register_mutex);
+	g_mutex_lock (&register_mutex);
 
 	file_source = g_object_new (file_source_type, NULL);
 	Main->priv->file_sources = g_list_append (Main->priv->file_sources, file_source);
 
-	g_static_mutex_unlock (&register_mutex);
+	g_mutex_unlock (&register_mutex);
 }
 
 
@@ -265,15 +265,29 @@ gth_main_get_all_file_sources (void)
 }
 
 
+#define MAX_ENTRY_POINTS_PER_SOURCE 1000
+
+
 GList *
 gth_main_get_all_entry_points (void)
 {
 	GList *list = NULL;
+	int    sort_order = 0;
 	GList *scan;
 
 	for (scan = gth_main_get_all_file_sources (); scan; scan = scan->next) {
 		GthFileSource *file_source = scan->data;
-		list = g_list_concat (list, gth_file_source_get_entry_points (file_source));
+		GList         *entry_points;
+		GList         *scan_entry;
+
+		entry_points = gth_file_source_get_entry_points (file_source);
+		for (scan_entry = entry_points; scan_entry; scan_entry = scan_entry->next) {
+			GthFileData *file_data = scan_entry->data;
+			g_file_info_set_sort_order (file_data->info, sort_order++);
+		}
+
+		list = g_list_concat (list, entry_points);
+		sort_order += MAX_ENTRY_POINTS_PER_SOURCE;
 	}
 
 	return list;
@@ -320,17 +334,17 @@ gth_main_register_metadata_category (GthMetadataCategory *metadata_category)
 {
 	int i;
 
-	g_static_mutex_lock (&register_mutex);
+	g_mutex_lock (&register_mutex);
 
 	for (i = 0; metadata_category[i].id != NULL; i++)
 		if (gth_main_get_metadata_category (metadata_category[i].id) == NULL)
 			g_ptr_array_add (Main->priv->metadata_category, &metadata_category[i]);
 
-	g_static_mutex_unlock (&register_mutex);
+	g_mutex_unlock (&register_mutex);
 }
 
 
-static GStaticMutex metadata_info_mutex = G_STATIC_MUTEX_INIT;
+static GMutex metadata_info_mutex;
 
 
 GthMetadataInfo *
@@ -341,14 +355,14 @@ gth_main_register_metadata_info (GthMetadataInfo *metadata_info)
 	if ((metadata_info->display_name != NULL) && (strstr (metadata_info->display_name, "0x") != NULL))
 		metadata_info->flags = GTH_METADATA_ALLOW_NOWHERE;
 
-	g_static_mutex_lock (&metadata_info_mutex);
+	g_mutex_lock (&metadata_info_mutex);
 
 	info = gth_metadata_info_dup (metadata_info);
 	g_ptr_array_add (Main->priv->metadata_info, info);
 	g_hash_table_insert (Main->priv->metadata_info_hash, (gpointer) info->id, info);
 	Main->priv->metadata_info_sorted = FALSE;
 
-	g_static_mutex_unlock (&metadata_info_mutex);
+	g_mutex_unlock (&metadata_info_mutex);
 
 	return info;
 }
@@ -359,7 +373,7 @@ gth_main_register_metadata_info_v (GthMetadataInfo metadata_info[])
 {
 	int i;
 
-	g_static_mutex_lock (&metadata_info_mutex);
+	g_mutex_lock (&metadata_info_mutex);
 
 	for (i = 0; metadata_info[i].id != NULL; i++)
 		if ((metadata_info[i].display_name == NULL) || (strstr (metadata_info[i].display_name, "0x") == NULL)) {
@@ -369,7 +383,7 @@ gth_main_register_metadata_info_v (GthMetadataInfo metadata_info[])
 			g_hash_table_insert (Main->priv->metadata_info_hash, (gpointer) (&metadata_info[i])->id, &metadata_info[i]);
 		}
 
-	g_static_mutex_unlock (&metadata_info_mutex);
+	g_mutex_unlock (&metadata_info_mutex);
 }
 
 
@@ -378,12 +392,12 @@ gth_main_register_metadata_provider (GType metadata_provider_type)
 {
 	GObject *metadata;
 
-	g_static_mutex_lock (&register_mutex);
+	g_mutex_lock (&register_mutex);
 
 	metadata = g_object_new (metadata_provider_type, NULL);
 	Main->priv->metadata_provider = g_list_append (Main->priv->metadata_provider, metadata);
 
-	g_static_mutex_unlock (&register_mutex);
+	g_mutex_unlock (&register_mutex);
 }
 
 
@@ -425,7 +439,7 @@ gth_main_get_metadata_attributes (const char *mask)
 	GList                  *scan;
 	char                  **values;
 
-	g_static_mutex_lock (&metadata_info_mutex);
+	g_mutex_lock (&metadata_info_mutex);
 
 	if (! Main->priv->metadata_info_sorted) {
 		g_ptr_array_sort (Main->priv->metadata_info, metadata_info_sort_func);
@@ -443,7 +457,7 @@ gth_main_get_metadata_attributes (const char *mask)
 	}
 	list = g_list_reverse (list);
 
-	g_static_mutex_unlock (&metadata_info_mutex);
+	g_mutex_unlock (&metadata_info_mutex);
 
 	values = g_new (char *, n + 1);
 	for (i = 0, scan = list; scan; i++, scan = scan->next)
@@ -539,7 +553,7 @@ gth_main_get_all_metadata_info (void)
 	GList *list = NULL;
 	int    i;
 
-	g_static_mutex_lock (&metadata_info_mutex);
+	g_mutex_lock (&metadata_info_mutex);
 
 	for (i = 0; i < Main->priv->metadata_info->len; i++) {
 		GthMetadataInfo *metadata_info = g_ptr_array_index (Main->priv->metadata_info, i);
@@ -548,7 +562,7 @@ gth_main_get_all_metadata_info (void)
 	}
 	list = g_list_reverse (list);
 
-	g_static_mutex_unlock (&metadata_info_mutex);
+	g_mutex_unlock (&metadata_info_mutex);
 
 	return list;
 }
@@ -557,9 +571,9 @@ gth_main_get_all_metadata_info (void)
 void
 gth_main_register_sort_type (GthFileDataSort *sort_type)
 {
-	g_static_mutex_lock (&register_mutex);
+	g_mutex_lock (&register_mutex);
 	g_hash_table_insert (Main->priv->sort_types, (gpointer) sort_type->name, sort_type);
-	g_static_mutex_unlock (&register_mutex);
+	g_mutex_unlock (&register_mutex);
 }
 
 
@@ -612,7 +626,6 @@ gth_main_get_all_sort_types (void)
 
 static GthTypeSpec *
 _gth_main_create_type_spec (GType       object_type,
-			    const char *first_property_name,
 			    va_list     var_args)
 {
 	GObject     *object;
@@ -622,17 +635,15 @@ _gth_main_create_type_spec (GType       object_type,
 
 	type_spec = gth_type_spec_new (object_type);
 
-	if (first_property_name == NULL)
-    		return type_spec;
-
     	object = g_object_new (object_type, NULL);
 
-    	type_spec->params = g_new (GParameter, n_alloced_params);
-  	name = first_property_name;
-  	while (name != NULL) {
+  	while ((name = va_arg (var_args, char *)) != NULL) {
 		GParamSpec *pspec;
 		char       *error = NULL;
 
+		if (type_spec->params == NULL)
+			type_spec->params = g_new (GParameter, n_alloced_params);
+
 		pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (object), name);
 		if (pspec == NULL) {
 			g_warning ("%s: object class `%s' has no property named `%s'",
@@ -652,7 +663,6 @@ _gth_main_create_type_spec (GType       object_type,
 			break;
 		}
 		type_spec->n_params++;
-		name = va_arg (var_args, char*);
 	}
 
 	g_object_unref (object);
@@ -661,30 +671,50 @@ _gth_main_create_type_spec (GType       object_type,
 }
 
 
+static void
+_register_image_loader_func (GthImageLoaderFunc  loader,
+			     GthImageFormat      native_format,
+			     const char         *mime_type)
+{
+	char *key;
+
+	key = g_strdup_printf ("%s-%d", mime_type, native_format);
+	g_hash_table_insert (Main->priv->image_loaders, (gpointer) key, loader);
+}
+
+
 void
 gth_main_register_image_loader_func (GthImageLoaderFunc  loader,
 				     GthImageFormat      native_format,
-				     const char         *first_mime_type,
 				     ...)
 {
 	va_list     var_args;
 	const char *mime_type;
 
-	g_static_mutex_lock (&register_mutex);
+	g_mutex_lock (&register_mutex);
 
-	va_start (var_args, first_mime_type);
-	mime_type = first_mime_type;
-  	while (mime_type != NULL) {
-  		char *key;
-
-  		key = g_strdup_printf ("%s-%d", mime_type, native_format);
-		g_hash_table_insert (Main->priv->image_loaders, (gpointer) key, loader);
-
-		mime_type = va_arg (var_args, const char *);
-  	}
+	va_start (var_args, native_format);
+  	while ((mime_type = va_arg (var_args, const char *)) != NULL)
+  		_register_image_loader_func (loader, native_format, mime_type);
 	va_end (var_args);
 
-	g_static_mutex_unlock (&register_mutex);
+	g_mutex_unlock (&register_mutex);
+}
+
+
+void
+gth_main_register_image_loader_func_v (GthImageLoaderFunc   loader,
+				       GthImageFormat       native_format,
+				       const char         **mime_types)
+{
+	int i;
+
+	g_mutex_lock (&register_mutex);
+
+	for (i = 0; mime_types[i] != NULL; i++)
+		_register_image_loader_func (loader, native_format, mime_types[i]);
+
+	g_mutex_unlock (&register_mutex);
 }
 
 
@@ -705,6 +735,8 @@ gth_main_get_image_loader_func (const char     *mime_type,
 	 * format. */
 
 	for (format = 0; (loader == NULL) && (format < GTH_IMAGE_N_FORMATS); format++) {
+		if (format == preferred_format)
+			continue;
 		g_free (key);
 		key = g_strdup_printf ("%s-%d", mime_type, format);
 		loader = g_hash_table_lookup (Main->priv->image_loaders, key);
@@ -716,23 +748,23 @@ gth_main_get_image_loader_func (const char     *mime_type,
 }
 
 
-GthPixbufSaver *
-gth_main_get_pixbuf_saver (const char *mime_type)
+GthImageSaver *
+gth_main_get_image_saver (const char *mime_type)
 {
 	GArray *savers;
 	int     i;
 
-	savers = gth_main_get_type_set ("pixbuf-saver");
+	savers = gth_main_get_type_set ("image-saver");
 	if (savers == NULL)
 		return NULL;
 
 	for (i = 0; i < savers->len; i++) {
 		GType           saver_type;
-		GthPixbufSaver *saver;
+		GthImageSaver *saver;
 
 		saver_type = g_array_index (savers, GType, i);
 		saver = g_object_new (saver_type, NULL);
-		if (gth_pixbuf_saver_can_save (saver, mime_type))
+		if (gth_image_saver_can_save (saver, mime_type))
 			return saver;
 
 		g_object_unref (saver);
@@ -838,7 +870,7 @@ gth_main_register_type (const char *set_name,
 {
 	GArray *set;
 
-	g_static_mutex_lock (&register_mutex);
+	g_mutex_lock (&register_mutex);
 
 	if (Main->priv->types == NULL)
 		Main->priv->types = g_hash_table_new_full (g_str_hash,
@@ -854,7 +886,7 @@ gth_main_register_type (const char *set_name,
 
 	g_array_append_val (set, object_type);
 
-	g_static_mutex_unlock (&register_mutex);
+	g_mutex_unlock (&register_mutex);
 }
 
 
@@ -876,7 +908,6 @@ void
 gth_main_register_object (GType       superclass_type,
 			  const char *object_id,
 			  GType       object_type,
-			  const char *first_property_name,
 			  ...)
 {
 	const char  *superclass_name;
@@ -886,7 +917,7 @@ gth_main_register_object (GType       superclass_type,
 	GthTypeSpec *spec;
 	char        *id;
 
-	g_static_mutex_lock (&register_mutex);
+	g_mutex_lock (&register_mutex);
 
 	if (object_id == NULL)
 		object_id = g_type_name (object_type);
@@ -917,15 +948,15 @@ gth_main_register_object (GType       superclass_type,
 		g_hash_table_insert (Main->priv->objects_order, g_strdup (superclass_name), object_order);
 	}
 
-	va_start (var_args, first_property_name);
-	spec = _gth_main_create_type_spec (object_type, first_property_name, var_args);
+	va_start (var_args, object_type);
+	spec = _gth_main_create_type_spec (object_type, var_args);
 	va_end (var_args);
 
 	id = g_strdup (object_id);
 	g_hash_table_insert (object_hash, id, spec);
 	g_ptr_array_add (object_order, id);
 
-	g_static_mutex_unlock (&register_mutex);
+	g_mutex_unlock (&register_mutex);
 }
 
 
@@ -1001,18 +1032,20 @@ gth_main_get_registered_object (GType       superclass_type,
 GBookmarkFile *
 gth_main_get_default_bookmarks (void)
 {
-	char *path;
+	GFile *file;
+	char  *filename;
 
 	if (Main->priv->bookmarks != NULL)
 		return Main->priv->bookmarks;
 
 	Main->priv->bookmarks = g_bookmark_file_new ();
 
-	path = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, BOOKMARKS_FILE, NULL);
-	g_bookmark_file_load_from_file (Main->priv->bookmarks,
-					path,
-					NULL);
-	g_free (path);
+	file = gth_user_dir_get_file_for_read (GTH_DIR_CONFIG, GTHUMB_DIR, BOOKMARKS_FILE, NULL);
+	filename = g_file_get_path (file);
+	g_bookmark_file_load_from_file (Main->priv->bookmarks, filename, NULL);
+
+	g_free (filename);
+	g_object_unref (file);
 
 	return Main->priv->bookmarks;
 }
@@ -1021,13 +1054,15 @@ gth_main_get_default_bookmarks (void)
 void
 gth_main_bookmarks_changed (void)
 {
-	char *filename;
+	GFile *file;
+	char  *filename;
 
-	gth_user_dir_make_dir_for_file (GTH_DIR_CONFIG, GTHUMB_DIR, BOOKMARKS_FILE, NULL);
-
-	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, BOOKMARKS_FILE, NULL);
+	file = gth_user_dir_get_file_for_write (GTH_DIR_CONFIG, GTHUMB_DIR, BOOKMARKS_FILE, NULL);
+	filename = g_file_get_path (file);
 	g_bookmark_file_to_file (Main->priv->bookmarks, filename, NULL);
+
 	g_free (filename);
+	g_object_unref (file);
 
 	gth_monitor_bookmarks_changed (gth_main_get_default_monitor ());
 }
@@ -1036,15 +1071,16 @@ gth_main_bookmarks_changed (void)
 GthFilterFile *
 gth_main_get_default_filter_file (void)
 {
-	char *path;
+	GFile *file;
 
 	if (Main->priv->filters != NULL)
 		return Main->priv->filters;
 
 	Main->priv->filters = gth_filter_file_new ();
-	path = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, FILTERS_FILE, NULL);
-	gth_filter_file_load_from_file (Main->priv->filters, path, NULL);
-	g_free (path);
+	file = gth_user_dir_get_file_for_read (GTH_DIR_CONFIG, GTHUMB_DIR, FILTERS_FILE, NULL);
+	gth_filter_file_load_from_file (Main->priv->filters, file, NULL);
+
+	g_object_unref (file);
 
 	return Main->priv->filters;
 }
@@ -1096,30 +1132,29 @@ gth_main_get_all_filters (void)
 void
 gth_main_filters_changed (void)
 {
-	char *filename;
-
-	gth_user_dir_make_dir_for_file (GTH_DIR_CONFIG, GTHUMB_DIR, FILTERS_FILE, NULL);
-	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, FILTERS_FILE, NULL);
-	gth_filter_file_to_file (Main->priv->filters, filename, NULL);
-
-	g_free (filename);
+	GFile *file;
 
+	file = gth_user_dir_get_file_for_read (GTH_DIR_CONFIG, GTHUMB_DIR, FILTERS_FILE, NULL);
+	gth_filter_file_to_file (Main->priv->filters, file, NULL);
 	gth_monitor_filters_changed (gth_main_get_default_monitor ());
+
+	g_object_unref (file);
 }
 
 
 GthTagsFile *
 gth_main_get_default_tag_file (void)
 {
-	char *path;
+	GFile *file;
 
 	if (Main->priv->tags != NULL)
 		return Main->priv->tags;
 
 	Main->priv->tags = gth_tags_file_new ();
-	path = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
-	gth_tags_file_load_from_file (Main->priv->tags, path, NULL);
-	g_free (path);
+	file = gth_user_dir_get_file_for_read (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
+	gth_tags_file_load_from_file (Main->priv->tags, file, NULL);
+
+	g_object_unref (file);
 
 	return Main->priv->tags;
 }
@@ -1135,14 +1170,13 @@ gth_main_get_all_tags (void)
 void
 gth_main_tags_changed (void)
 {
-	char *filename;
+	GFile *file;
 
-	gth_user_dir_make_dir_for_file (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
-	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
-	gth_tags_file_to_file (Main->priv->tags, filename, NULL);
+	file = gth_user_dir_get_file_for_read (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
+	gth_tags_file_to_file (Main->priv->tags, file, NULL);
 	gth_monitor_tags_changed (gth_main_get_default_monitor ());
 
-	g_free (filename);
+	g_object_unref (file);
 }
 
 
@@ -1199,7 +1233,6 @@ gth_main_activate_extensions (void)
 						"photo_importer",
 						"photobucket",
 						"picasaweb",
-						"pixbuf_savers",
 						"raw_files",
 						"red_eye_removal",
 						"rename_series",
